cd c:\Python\ssdse\simple1
# simple1.py
#-------------------------------------------------------------------
# SSDSEのデータを使用して、基本的な分析処理を行う
# 分析目標は、生産年齢人口に対する、1000人当たりの状況を調査する
# 
#< ssdse.csv:			教育用標準データセット（基本データ）
#> simple1_mean.csv")	生産者年齢人口1000人当たりの各平均値
#> simple1_base.pickle	データ分析対象のデータをデータフレームで保存
#> simple1.pickle		データ分析対象の処理結果をデータフレームで保存
#-------------------------------------------------------------------
# 2019/05/13:	新規作成
#-------------------------------------------------------------------
# 予め、Pythonで使用するライブラリを呼び出しておく
import pandas as pd
import numpy as np

#-------------------------------------------------------------------
# １．データの読み込み（データは、1行目にヘッダー情報（変数情報）が付されている）
FILE = "../data/ssdse.csv"  # 相対パスで指定（..は、カレントフォルダの1つ上位のフォルダを示す）
COLS = [0,1,10,20,21,32,33,88,111]  # 使用する列番号は、1つ目を0から数えて指定
# DataFrameの名前は、dfとせず、独自の名前を付けないと区別がつかない
ssdse = pd.read_csv(FILE,usecols=COLS,encoding="Shift-JIS")  # ssdseというDF（データフレーム）として読み込む
ssdse  # ssdseのDFの内容が表示される。0から始まる数字がindex（行の識別名）として自動的に付される。
# 1741件（indexsは、0～1740となっている）
ssdse.head(3).T  # 全変数のhead(3)最初の3件を確認(.Tを付すことで転置（行と列を入れ替え）して表示される)
ssdse.tail(3).T  # 全変数の最後の3件を確認
# 本来、変数名は小文字で定義するのであるが、SSDSEから提供されているファイルのヘッダーには、大文字であるため
# そのまま大文字の変数名となっている。通常、大文字で定義するのは、定数等であって、変数には使用しない。
# 1)mcode:市町村コード・・・この変数のみ新規に定義したので小文字で設定している
# 2)A1101:人口総数
# 3)A1302:15～64歳人口（生産者年齢人口）
# 4)A4101:出生数
# 5)A4200:死亡数
# 6)A9101:婚姻件数
# 7)A9201:離婚件数
# 8)E1501:幼稚園在園者数
# 9)J2506:保育所等在所児数
len(ssdse)  # データのカウント（lenは、長さを測る）
# 1741件であることがわかる。
ssdse?
# ssdseがDataFrameオブジェクトであることが分かる。?を付けてIPythonに表示させる場合は、同一行にコメントを付せないので注意
# データ内容をssdse.csvをテキストエディタで開いて確認する。
#-------------------------------------------------------------------
#２．データの確認・・・予め定義されたメソッド（DataFrameに定義された操作）を活用する。
#
# DFataFrameに角括弧で変数名を指定し、メソッドに丸括弧で操作の引数（引数が無い場合も丸括弧は必要）を指定する。
ssdse["mcode"].describe()  # 変数に続けて、ドットでメソッドのdescribe()を指定する。
# mcodeの基本統計量（件数、平均、標準偏差、最小値、第1四分位、中央値、第2四分位、最大値）とデータタイプを表示する。
# mcodeは整数（int）の値であるが、データタイプを指定しないと実数（float）として読み込まれる。
# 但し、欠測値が含まれる場合は、整数と指定しても実数として扱われる。
# ここでは、実数でも問題ないのでこのままで良いものとする。
# 市町村コードは、1100～47381であることが分かる。
# 下3桁が、都道府県内の番号で、その上位桁が都道府県（1～47）を表している。

# 他の変数も確認
ssdse["A1101"].describe()  # 人口総数の基本統計量
# 最大人口は、3,724,844人であることが分かる
ssdse[ssdse["A1101"] == 3724844]["mcode"]  # 最大人口に一致する行の市町村コードを確認
# indexが689番の市町村コード14100（横浜市）であることが分かる。
# 最小人口が0人の市町村があることも分かる。
ssdse[ssdse["A1101"] == 0]["mcode"]  # 人口総数が0人の市町村コードを確認
# 7543、7545、7546、7547の4市町村であったことが分かる。いずれも福島県の町村であった。
# 2011年3月11日の東日本大震災発生後の2015年のデータであるためと考えられる。
#人口が0件のデータは分析対象ではないので削除したデータフレームを新たに作成する
ssdse2 = ssdse[ssdse["A1101"] > 0]  # 人口総数が0より大きいデータを抽出する
ssdse2["A1101"].describe()  # 新しいデータフレームの人口総数の基本統計量
#　最小人口が18名となる4件少ない、1737件のデータができていることが分かる。

ssdse2["A1302"].describe()  # 生産者年齢人口の基本統計量
# 生産年齢人口が0人のデータが存在することが分かる
ssdse2[ssdse2["A1302"] == 0]["mcode"]  # 生産年齢人口が0人の市町村コードを確認
# 7564の1市町村であったことが分かる。これもも福島県の町村であった。
ssdse[ssdse["A1302"] == 0]["A1101"]  # 生産年齢人口が0人の人口総数を確認
# 7564の人口総数は41人であったことが分かる。
# 分析目標から考えて、生産年齢人口が100人未満の市町村は、分析対象から除外することにする
ssdse3 = ssdse[ssdse["A1302"] > 99]  # 清算人口が99人より大きいデータを抽出する
ssdse3["A1302"].describe()  # 生産者年齢人口の基本統計量
# 生産年齢人口が最低130人の1735件（元データから6件減った）の新しいデータができた

ssdse3["A4101"].describe()  # 出生数の基本統計量
# 出生数が0件のデータがった
ssdse3[ssdse3["A4101"] == 0]["mcode"]  # 出生数が0人の市町村コードを確認
# 7446、29444、29451の3市町村であったことが分かる。福島県と奈良県の町村であった。
# また、29444、29451のindexは、1227、1232であった。
ssdse[1227:1233]  # 行番号の1227から1233の前までのデータを元データから確認する
ssdse3.loc[1227:1232]  # ssdse3は行番号に6件のずれがあるので、行ラベルで参照するlocを使うと同じ結果になる
# 但し、行番号では参照範囲のエンドを次の番号で指定したが、参照する最後の行ラベルで指定する
# 29444、29451の生産者年齢人は、301人と243人であった。
# データ分析対象の生産年齢人口を1000人以上とすることにする。
ssdse4 = ssdse[ssdse["A1302"] > 999]  # 清算人口が999人より大きいデータを抽出する
ssdse4["A1302"].describe()  # 生産年齢人口の基本統計量
1741 - 1652
# 生産年齢人口が最低1016人の1652件（元データから89件減った）の新しいデータができた
ssdse4["A4101"].describe()  # 出生数の基本統計量
# 出生数の最小値は、5人となった

ssdse4["A4200"].describe()  # 死亡数の基本統計量
# 死亡数の最小値は、10人であった

ssdse4["A9101"].describe()  # 婚姻件数の基本統計量
# 婚姻数の最小値は、2人であった

ssdse4["A9201"].describe()  # 離婚件数の基本統計量
# 離婚件数の最小値に、0人があった
ssdse4[ssdse4["A9201"] == 0]["mcode"]  # 離婚件数が0人の市町村コードを確認
# 1455、1462、1511、5464、20446、35341、45430の7市町村が該当した。
# それぞれの行ラベルは、89、96、112、311、888、1368、1652であった
ssdse4.loc[[89,96,112,311,888,1368,1652]]  # 複数の行ラベルを指定して確認
# この中の生産年齢人口では、1455の1896人が一番多かった
ssdse5 = ssdse[ssdse["A1302"] > 1999]  # 生産年齢人口が2000人以上のデータを抽出する
ssdse5["A9201"].describe()  # 離婚件数の基本統計量
# 離婚件数の最小値は、1人であった

ssdse5["E1501"].describe()  # 幼稚園在園者数の基本統計量
# 幼稚園在園者数の最小値に0人があった
ssdse5[ssdse5["E1501"] == 0]["mcode"]  # 幼稚園在園者数が0人の市町村コードを確認
# 297市町村で0人であった。こういうデータであったものとして進める

ssdse5["J2506"].describe()  # 保育所等在所児数の基本統計量
# 保育所等在所児数の最小値に0人があった
ssdse5[ssdse5["J2506"] == 0]["mcode"]  # 保育所等在所児数が0人の市町村コードを確認
# 19市町村で0人であった。しかし、幼稚園在園者数が0人の市町村とは重なってい無さそうである。
ssdse5["enji_jidou"] = ssdse5["E1501"] + ssdse5["J2506"]  # 園児と児童の合計数を計算
ssdse5["enji_jidou"].describe()  # 園児・児童数の基本統計量
# 園児・児童数の最小値に0人があった
ssdse5[ssdse5["enji_jidou"] == 0]["mcode"]  # 園児・児童数が0人の市町村コードを確認
# 6361、20324、20349、20407、20411、23427の6市町村で0人であった。
# 行ラベルは、332、857、858、872、875、1028であった。
ssdse5.loc[[332,857,858,872,875,1028]]  # 複数の行ラベルを指定して確認
# この中の生産年齢人口では、20324の4343人が一番多かった
ssdse6 = ssdse5[ssdse5["A1302"] >= 5000]  # 生産年齢人口が5000人以上のデータを抽出する
# 園児・児童数の値があるssdse5から作成
ssdse6["A1302"].describe()  # 生産年齢人口の基本統計量
1741 - 1259
# 生産年齢人口が最低5001人の1259件（元データから482件減った）の新しいデータができた
# 本データを分析対象とする

#-------------------------------------------------------------------
#３．データの分析・・・分析対象となった市町村のデータで処理する
#
ssdse6  # データの確認
ssdse6.head(5).T  # 転置した最初の5件を確認

simple1 = pd.DataFrame(ssdse6["mcode"])  # simple1という市町村コードだけのデータフレームを定義
simple1["prod"] = ssdse6["A1302"] / ssdse6["A1101"]  # 人口総数に対する生産年齢人口の割合を設定
simple1["birth"] = ssdse6["A4101"] / ssdse6["A1302"] * 1000  # 生産年齢人口1千人当たり出生数を設定
simple1["death"] = ssdse6["A4200"] / ssdse6["A1302"] * 1000  # 生産年齢人口1千人当たり死亡数を設定
simple1["marri"] = ssdse6["A9101"] / ssdse6["A1302"] * 1000  # 生産年齢人口1千人当たり婚姻数を設定
simple1["divor"] = ssdse6["A9201"] / ssdse6["A1302"] * 1000  # 生産年齢人口1千人当たり離婚数を設定
simple1["child"] = ssdse6["enji_jidou"] / ssdse6["A1302"] * 1000  # 生産年齢人口1千人当たり園児・児童数を設定
simple1 = simple1.reset_index(drop=True)  # indexを再設定することで、0からの連続した値になり、行番号と行ラベルが一致する
simple1_data = simple1.drop("mcode",axis=1)  # 市町村コードを除いたデータフレームを作成
simple1  # 確認する
simple1_mean = simple1_data.mean()  # 各データの平均値を求める
simple1_mean  # 確認する
simple1_mean.to_csv("simple1_mean.csv")  # 平均値のデータをcsvファイルとして出力

#-------------------------------------------------------------------
#４．データの保存・・・分析処理した結果データを保存
#
simple1_base = ssdse6.drop(["E1501","J2506"],axis=1)  # 園児数と児童数の変数を削除して保存

# 処理結果のデータフレームをデータフレームの形式で保存する
# 最初に%timeを付すことで、実行時間を測ることができる
%time simple1_base.to_pickle("simple1_base.pickle") # データフレームをpickle形式で保存
# 0 ns
%time simple1.to_pickle("simple1.pickle") # データフレームをpickle形式で保存
# 78.1 ms
